package interceptors;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.Interceptor;

import pda.IMain;
import printer.IPrinter;

public class PrinterLifeCycleInterceptor implements IPrinter, LifeCycleController, Interceptor{

	public IPrinter delegate;
	private boolean started;
	private int counter;
	
	
	
	
	
	public String getFcState() {
	   return started ? STARTED : STOPPED;
	}

	public void startFc() throws IllegalLifeCycleException {
		
		System.out.println("[Printer lifecycle] started");
		started = true;
		notifyAll();
		
	}

	public void stopFc() throws IllegalLifeCycleException {
		while (counter > 0) { try { wait(); } catch (InterruptedException _) {} }
	  started = false;
	}
	
	
	
	

	
	
	public String print(String str) throws NoSuchInterfaceException {
		 synchronized (this) {
	      while (counter == 0 && !started) { try { wait(); } catch (InterruptedException _) {} }
	      ++counter;
	    }
	    try {
	    
	    	return delegate.print(str);
	  
	    } finally {
	      synchronized (this) {
	        --counter;
	        if (counter == 0) { notifyAll(); }
	      }
	    }
	}


	public void init() {
		 synchronized (this) {
	      while (counter == 0 && !started) { try { wait(); } catch (InterruptedException _) {} }
	      ++counter;
	    }
	    try {
	    
	    	delegate.init();
	  
	    } finally {
	      synchronized (this) {
	        --counter;
	        if (counter == 0) { notifyAll(); }
	      }
	    }
	} 
	
	
	
	
	
	

	public void start() {
		synchronized (this) {
      while (counter == 0 && !started) { try { wait(); } catch (InterruptedException _) {} }
      ++counter;
    }
    try {
    
    	delegate.start();
  
    } finally {
      synchronized (this) {
        --counter;
        if (counter == 0) { notifyAll(); }
      }
    }
		
	}

	public void stop() throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		synchronized (this) {
      while (counter == 0 && !started) { try { wait(); } catch (InterruptedException _) {} }
      ++counter;
    }
    try {
    
    	delegate.stop();
  
    } finally {
      synchronized (this) {
        --counter;
        if (counter == 0) { notifyAll(); }
      }
    }
		
	}
	
	
	public Object getFcItfDelegate() {
		return delegate; 
	}

	public void setFcItfDelegate(Object delegate) {
	  this.delegate = (IPrinter) delegate;
		
	}

	public void initFcController(InitializationContext arg0) throws InstantiationException {
	}
	
	/**
   * 
   */
	public Object clone() {
    try {
      return super.clone();
    }
    catch( CloneNotSupportedException cnse ) {
    }
    return null;
	}


}
