package registry;




import java.util.ArrayList;
import java.util.Iterator;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.ComponentInterface;
import org.objectweb.fractal.julia.Interceptor;
import org.objectweb.fractal.juliak.control.interceptor.InterceptorController;
import org.objectweb.fractal.util.Fractal;

import pda.AmbientInterceptor;
import pda.IMain;
import printer.IPrinter;

import run.IAmbientController;




public class RegistryImpl_old implements IRegistry {

	

	//list of bindings ???
	ArrayList bindings;
	
	//two lists of services:
	ArrayList provServices;
	ArrayList reqServices;
	
	
	public RegistryImpl_old() {
		provServices = new ArrayList();
		bindings = new ArrayList();
		reqServices = new ArrayList();
	}
	
  /**
   * @throws IllegalLifeCycleException 
   * @throws IllegalBindingException 
   * @throws NoSuchInterfaceException 
   * 
   * 
   * 
   * 
   * 
   */
	public void registerService(Component comp, String providedService, String requiredService) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		
		if (providedService != null) { 
			Service srv = new Service(comp,providedService);
		  if (isNotAlreadyBind(srv)) { 
		    provServices.add(srv);
			  System.out.println("[REGISTER] Provided Service Registered: " + srv.getService());
			  findBindingForProvidedService(srv);
		  }
		  
		}
		 
		if (requiredService != null) { 
		  Service srv = new Service(comp,requiredService);
		  
		  reqServices.add(srv);
		  
			System.out.println("[REGISTER] Required Service Registered: " + srv.getService());
      
		  
		  findBindingForRequiredService(srv);
		}
		
	}

	/**
	 * 
	 * @param comp
	 * @param service
	 * @throws NoSuchInterfaceException 
	 */
	public void reactivateService(Component comp, String service) throws NoSuchInterfaceException {
		
		//find the binding
		AmbientBinding bnd = getBinding(comp,service);
		
		
		
		//reactivate the binding
		
		
		IAmbientController ambCont = (IAmbientController) bnd.getClient().getFcInterface("IAmbientController");
		InterceptorController intCont = (InterceptorController) bnd.getClient().getFcInterface("/interceptor-controller");
	 	ComponentInterface itf = (ComponentInterface) bnd.getClient().getFcInterface(service);
	 	ComponentInterface provItf = (ComponentInterface) bnd.getServer().getFcInterface(service);
		
		
	 	
		//obtaining the interceptors
		Interceptor[] interceptorList = intCont.getFcInterceptors(itf);
		
	  // TODO: going through the list of interceptors		
		Interceptor interceptor = interceptorList[0];
		interceptor.setFcItfDelegate(provItf);
		
		System.out.println("[REGISTER] Ambient Binding reactivated for the service: " + service);
		
	}

	/**
	 * 
	 * @param comp
	 * @param service
	 * @return
	 */
	private AmbientBinding getBinding(Component comp, String service) {
	
		Iterator iter = bindings.iterator();
		
		while(iter.hasNext()) {
	    AmbientBinding bnd = (AmbientBinding) iter.next();
	    
	    if (bnd.getService().equals(service)&&
	   		 bnd.getServer().equals(comp))
	     	return bnd;
	  }
		
		return null;
	}
 
  /**
   * 
   * 
   * @param srv
   * @return
   * @throws NoSuchInterfaceException 
   */
  private boolean isNotAlreadyBind(Service srv) throws NoSuchInterfaceException {
		
  	Iterator iter = bindings.iterator();
		
		 while(iter.hasNext()) {
	     AmbientBinding bnd = (AmbientBinding) iter.next();
	     
	     if (bnd.getService().equals(srv.getService())&&
	    		 bnd.getServer().equals(srv.getController()))
	      
	    	 //binding already exists, we will activate it again
	    	 reactivateAmbientBinding(bnd);
	    	 
	    	 return false;
	   }
  	
  	return true;
	}

	/**
   * @throws NoSuchInterfaceException 
   * @throws IllegalLifeCycleException 
   * @throws IllegalBindingException 
   * 
   * 
   * 
   * 
   * 
   */
	public void unregisterProvService(Component comp) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		Iterator iter = provServices.iterator();
		
		 while(iter.hasNext()) {
	      Service srv = (Service) iter.next();
	     
	      if (srv.getController().equals(comp)) {
	      	provServices.remove(srv);
	      	
	      	removeProvAmbientBinding(srv);
	      	//suspendAmbientBinding(client, service)
	      	System.out.println("[Registry] Unregistering the service: " + srv.getService());
	      	
	      	break;
	      }
	   }
	}

	
	
	
	
	
  /**
   * 
   * 
   * 
   * 
   */
	public void unregisterReqService(Component comp) {
		Iterator iter = reqServices.iterator();
		
		 while(iter.hasNext()) {
	      Service srv = (Service) iter.next();
	     
	      if (srv.getController().equals(comp)) {
	      	reqServices.remove(srv);
	      	break;
	      }
	   }
		
	}

	
	
	
	
	/**
	 * 
	 * ambient reference must be preserved even after removal of the server interface
	 * the client component has to be informed to store the messages during the time
	 * when the server component is not available.
	 * 
	 * @param srv
	 * @throws NoSuchInterfaceException 
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalBindingException 
	 */
	private void removeProvAmbientBinding(Service srv) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		Iterator iter = bindings.iterator();
		
		 while(iter.hasNext()) {
	      AmbientBinding bind = (AmbientBinding) iter.next();
	     
	      if (srv.getController().equals(bind.getServer())) {
	      	//reqServices.remove(srv);
	      	
	      	//inform the client compoentn
	      	//removeAmbientBinding(bind.getClient(),bind.getService());
	      	suspendAmbientBinding(bind.getClient(),bind.getService());
	      	
	      	break;
	      }
	   }
		
	}
	
	
	
	/**
	 * 
	 * Removing client component - binding is deleted and nothing more is needed
	 * 
	 * @param srv
	 */
	private void removeReqAmbientBinding(Service srv) {
		Iterator iter = bindings.iterator();
		
		 while(iter.hasNext()) {
	      AmbientBinding bind = (AmbientBinding) iter.next();
	     
	      if (srv.getController().equals(bind.getClient())) {
	      	bindings.remove(bind);
	      	break;
	      }
	   }
		
	}
	
	
	
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////// Creating ambient bindings
	
	
	
	
	/**
	 * 
	 * @param srv
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalBindingException 
	 * @throws NoSuchInterfaceException 
	 */
	private void findBindingForProvidedService(Service srv) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
			
		Iterator iter = reqServices.iterator();
		
		 while(iter.hasNext()) {
	      Service req = (Service) iter.next();
	     
	      if (req.getService().equals(srv.getService())) {
	      	
	      	//req.getController().bindAmbientFc(srv.getService(),srv.getController());
	      	createAmbientBinding(req.getController(),srv.getService(),srv.getController());
	      	
	      	AmbientBinding binding = new AmbientBinding(srv.getService(),req.getController(),srv.getController());
	      	bindings.add(binding);
	      	
	      	System.out.println("[REGISTER] I have find an ambient binding for the service: " + srv.getService());
	      	
	      	break;
	      }
	   }
		
	}  
	
	
	/**
	 * 
	 * @param srv
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalBindingException 
	 * @throws NoSuchInterfaceException 
	 */
	private void findBindingForRequiredService(Service srv) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
			
		Iterator iter = provServices.iterator();
		
		 while(iter.hasNext()) {
	      Service prov = (Service) iter.next();
	     
	      if (prov.getService().equals(srv.getService())) {
	      	
	      	//create ambient bindings
	      	createAmbientBinding(srv.getController(),srv.getService(),prov.getController());
	      
	      	AmbientBinding binding = new AmbientBinding(srv.getService(),srv.getController(),prov.getController());
	      	bindings.add(binding);
	      	
	      	break;
	      }
	   }
		
	}

	
	
	
	
	/**
	 * 
	 * @param client
	 * @param service
	 * @throws NoSuchInterfaceException 
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalBindingException 
	 */
	private void createAmbientBinding(Component client, String service, Component server) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		
		IAmbientController ambCont = (IAmbientController) client.getFcInterface("iAmbientController");
		InterceptorController interceptorController = (InterceptorController) client.getFcInterface("/interceptor-controller");
		ComponentInterface myInterface = (ComponentInterface) client.getFcInterface(service);
   	//bind.getClient().unbindAmbientFc(bind.getService());

		ComponentInterface providedInterface = (ComponentInterface) server.getFcInterface(service);
		
   	//binding
		//ambCont.bindFc(service, providedInterface);
	  //	
		//IMain main = (IMain) client.getFcInterface("iMain");
		//main.getIPrinter().print("aaaaaaaaaaassdf");
		
		Fractal.getBindingController(client).bindFc(service, providedInterface);
		
		
		//creating interceptor
		Interceptor myNewInterceptor = (Interceptor) new AmbientInterceptor();
		
    //add interceptor
		interceptorController.addFcInterceptor(myInterface, myNewInterceptor);
		
		
		
		
		
	}
	
	
	/**
	 * 
	 * @param client
	 * @param service
	 * @throws NoSuchInterfaceException 
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalBindingException 
	 */
	private void removeAmbientBinding(Component client, String service) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		
		//obtaining controller
		IAmbientController ambCont = (IAmbientController) client.getFcInterface("iAmbientController");
		
		//removing the binding
		ambCont.unbindFc(service);
		
	}
	
	/**
	 * 
	 * Will set the field "delegate" inside the AmbientInterceptor to null.
	 * 
	 * @param client
	 * @param service
	 * @throws NoSuchInterfaceException 
	 */
	private void suspendAmbientBinding(Component client, String service) throws NoSuchInterfaceException {
		
		IAmbientController ambCont = (IAmbientController) client.getFcInterface("iAmbientController");
		InterceptorController intCont = (InterceptorController) client.getFcInterface("/interceptor-controller");
	 	ComponentInterface itf = (ComponentInterface) client.getFcInterface(service);
		
		
		//obtaining the interceptor
		Interceptor[] interceptorList = intCont.getFcInterceptors(itf);
		
	  // TODO: going through the list of interceptors		
		Interceptor interceptor = interceptorList[0];
		interceptor.setFcItfDelegate(null);
		
		System.out.println("[REGISTER] Ambient Binding suspended for the service: " + service);
    
		
	}
	
	/**
	 * 
	 * @param client
	 * @param service
	 * @throws NoSuchInterfaceException 
	 */
	private void reactivateAmbientBinding(AmbientBinding binding) throws NoSuchInterfaceException {
		
		Component client = binding.getClient();
		IAmbientController ambCont = (IAmbientController) client.getFcInterface("iAmbientController");
		InterceptorController intCont = (InterceptorController) client.getFcInterface("/interceptor-controller");
	 	ComponentInterface itf = (ComponentInterface) client.getFcInterface("iPrinter");
	 	ComponentInterface provItf = (ComponentInterface) binding.getServer().getFcInterface("iPrinter");
		
		
		//obtaining the interceptor
		Interceptor[] interceptorList = intCont.getFcInterceptors(itf);
		
	  // TODO: going through the list of interceptors		
		Interceptor interceptor = interceptorList[0];
		interceptor.setFcItfDelegate(provItf);
		
		System.out.println("[REGISTER] Ambient Binding activated again for the service: " + binding.getService());
    
		
	}
	
	
}


