package run;

import interceptors.PDALifeCycleInterceptor;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.julia.ComponentInterface;
import org.objectweb.fractal.juliak.FractalHelper;
import org.objectweb.fractal.juliak.control.interceptor.InterceptorController;
import org.objectweb.fractal.juliak.control.membrane.MembraneController;
import org.objectweb.fractal.juliak.membrane.CustomFactory;
import org.objectweb.fractal.util.Fractal;

import ambient.AmbientControllerImpl;
import ambient.AmbientPrimitive;

import pda.IAttributeManager;
import pda.IMain;
import printer.IPrinter;

public class AmbientReferenceExample {

	
	Component mainComp = null;
	Component pdaComp = null;
	Component printerComp = null;
	
	Component registryComp = null;
	
	/**
	 * 
	 * @throws InstantiationException
	 * @throws NoSuchInterfaceException
	 * @throws ClassNotFoundException
	 * @throws IllegalAccessException
	 * @throws java.lang.InstantiationException
	 * @throws IllegalBindingException 
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalContentException 
	 */
	public void instantiate() throws InstantiationException, NoSuchInterfaceException, ClassNotFoundException, IllegalAccessException, java.lang.InstantiationException, IllegalContentException, IllegalLifeCycleException, IllegalBindingException {
		
		
    // creating all the components 
		createComponents();
		
		//components assembly
		componentAssembly();
		
		
	}
	
	
	
	
	/**
	 * @throws NoSuchInterfaceException 
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalBindingException 
	 * 
	 *
	 */
	public void runExample(String[] args) throws IllegalLifeCycleException, NoSuchInterfaceException, IllegalBindingException {
		

		
		
		// -------------------------------------------------------------------
    // 3. RUNNING
    // -------------------------------------------------------------------

    
		//LifeController
		//((LifeCycleController) mainComp.getFcInterface("lifecycle-controller")).startFc();
		Fractal.getLifeCycleController(mainComp).startFc();
    
		    
		//initialization of the components
	  ((IMain) mainComp.getFcInterface("iMain")).init();
	  ((IPrinter) printerComp.getFcInterface("iPrinter")).init();
		
	  
	  System.out.println("\n\n...running");
     //	 and run 
	  ((IMain) mainComp.getFcInterface("iMain")).main(args);
		
		System.out.println("\n\n");
		
		
    /////////////////////////////////////////////////////////////////////////////
    
    // stoping the printer
	  ((IPrinter) printerComp.getFcInterface("iPrinter")).stop();
		
	  System.out.println("\n\n");
	  
	  //	 and run 
	  ((IMain) mainComp.getFcInterface("iMain")).main(args);
		((IPrinter) printerComp.getFcInterface("iPrinter")).start();
    
   
    
    
    
    
    
    /////////////////////////////////////////////////////////////////////////////
   
   
    //
    // Re-invoking the component.
    //
    //((Runnable)itf).run();
    //((IMain) mainComp.getFcInterface("iMain")).main(args);
    

    
    
		
		
	}
	
	
	
	
	private void createComponents() throws InstantiationException, NoSuchInterfaceException, ClassNotFoundException, IllegalAccessException, java.lang.InstantiationException, IllegalLifeCycleException {
	  
		//	BootStraping
		Component boot = Fractal.getBootstrapComponent();
    TypeFactory tf = Fractal.getTypeFactory(boot);
    GenericFactory gf = Fractal.getGenericFactory(boot);
    
		  	
		ComponentType mainType = tf.createFcType( new InterfaceType[]
		           {tf.createFcItfType("iMain","pda.IMain",false,false,false)}
		);
		
		ComponentType cType = tf.createFcType( new InterfaceType[]{ 
				tf.createFcItfType("iMain","pda.IMain",false,false,false),
				tf.createFcItfType("iPrinter","printer.IPrinter",true,true,false),  // ambient reference!
				tf.createFcItfType("iFutureCallback","ambient.IFutureCallback",false,false,false),
				tf.createFcItfType("iRegistry","registry.IRegistry",true,false,false),
				tf.createFcItfType("attribute-controller","pda.IAttributeManager",false,false,false),
						
		});
		                                      		
		ComponentType sType = tf.createFcType( new InterfaceType[]{ 
				tf.createFcItfType("iPrinter","printer.IPrinter",false,false,false),
				tf.createFcItfType("iRegistry","registry.IRegistry",true,false,false),
				tf.createFcItfType("iFutureCallback","ambient.IFutureCallback",true,false,false),
				tf.createFcItfType("attribute-controller","pda.IAttributeManager",false,false,false)	
		});
				
		ComponentType registryType = tf.createFcType( new InterfaceType[]{ 
				tf.createFcItfType("iRegistry","registry.IRegistry",false,false,false),
		}); 
	
		
		//			 -------------------------------------------------------------------
    // 2. CREATE COMPONENTS DIRECTLY
    // -------------------------------------------------------------------

    
		
		//
		// -------------------------------------------------------------------
    // create root component
    // -------------------------------------------------------------------
   
		mainComp = gf.newFcInstance(mainType, "composite", null);

    
    
    // create client component
    //pdaComp = gf.newFcInstance(cType, "primitive", "pda.PDAImpl");
    

		//	 -------------------------------------------------------------------
    // PDA Component
    // -------------------------------------------------------------------
    
    pdaComp = createComponentWithAmbientMembrane(cType, pda.PDAImpl.class.getName() ,gf,tf);
    // Membrane Setting for PDAComp
    Object itf = pdaComp.getFcInterface("/membrane-controller");
    MembraneController mc = (MembraneController) itf;
    Component membrane = mc.getFcMembrane();
    
    setAmbientController(membrane);
    
    ((IAttributeManager)Fractal.getAttributeController(pdaComp)).setComponent(pdaComp);
    
    
    
    /*
     *  lifecycle interceptor
     */ /*
    InterceptorController ic = (InterceptorController) membrane.getFcInterface("///interceptor-controller");
		// the interface of the client component
		ComponentInterface myInterface = (ComponentInterface) pdaComp.getFcInterface("iMain");
	  PDALifeCycleInterceptor pdaLC = new PDALifeCycleInterceptor();
		ic.addFcInterceptor(myInterface, pdaLC);
		
		//lifecycle inteceptor
		InterceptorController mic = (InterceptorController) membrane.getFcInterface("interceptor-controller");
		
		ComponentInterface lfIntf = (ComponentInterface) membrane.getFcInterface("//lifecycle-controller");
		mic.addFcInterceptor(lfIntf, pdaLC);
		*/
		

    
    
    
    
    
    
    
    
    
    
    
    //	 -------------------------------------------------------------------
    // PrinterComp Component
    // -------------------------------------------------------------------
   
    //Component c2 = gf.newFcInstance(type, "ambientPrimitive", C.class.getName());
    // create server component
    //printerComp = gf.newFcInstance(sType, "primitive", printer.PrinterImpl.class.getName());
    
    
    printerComp = createComponentWithAmbientMembrane(sType,printer.PrinterImpl.class.getName(),gf,tf);
    // Membrane Setting for PDAComp
    Object itf2 = printerComp.getFcInterface("/membrane-controller");
    MembraneController mc2 = (MembraneController) itf2;
    Component membrane2 = mc2.getFcMembrane();
    
    setAmbientController(membrane2);
    
    ((IAttributeManager)Fractal.getAttributeController(printerComp)).setComponent(printerComp);
    
    
    
    
    
    
    
    
    //	 -------------------------------------------------------------------
    // creating registry Component
    // -------------------------------------------------------------------
    registryComp = gf.newFcInstance(registryType, "primitive", "registry.RegistryImpl");
    
    
   
    
    
	}
	
	
	/**
	 * 
	 * @throws IllegalContentException
	 * @throws IllegalLifeCycleException
	 * @throws NoSuchInterfaceException
	 * @throws IllegalBindingException
	 */
	private void componentAssembly() throws IllegalContentException, IllegalLifeCycleException, NoSuchInterfaceException, IllegalBindingException {
		
		

    // -------------------------------------------------------------------
    // 3. component Assembly
    // -------------------------------------------------------------------

    
    Fractal.getContentController(mainComp).addFcSubComponent(pdaComp);
    Fractal.getContentController(mainComp).addFcSubComponent(printerComp);
    Fractal.getContentController(mainComp).addFcSubComponent(registryComp);
    Fractal.getBindingController(mainComp).bindFc("iMain", pdaComp.getFcInterface("iMain"));
    
    
    //binding to Registry
    Fractal.getBindingController(pdaComp).bindFc("iRegistry", registryComp.getFcInterface("iRegistry"));
    Fractal.getBindingController(printerComp).bindFc("iRegistry", registryComp.getFcInterface("iRegistry"));
    
       
    
    
    
		
	}
	
	
	
	
	
	
	
	
	

	/**
	 * 
	 * @param tf 
	 * @param gf 
	 * @param type 
	 * @return
	 * @throws ClassNotFoundException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws InstantiationException 
	 * @throws java.lang.InstantiationException 
	 * @throws NoSuchInterfaceException 
	 */
	private static Component createComponentWithAmbientMembrane(ComponentType cType,String componentClass ,GenericFactory gf, TypeFactory tf) throws ClassNotFoundException, InstantiationException, IllegalAccessException, InstantiationException, java.lang.InstantiationException, NoSuchInterfaceException {


		//Creating a new Membrane
    
   	//String adl = AmbientPrimitive.class.getName();
	  String adl = AmbientPrimitive.class.getName();
	   
    /*
     * Use a component as the control membrane for creating another
     * component.
     */
    Class cl = Class.forName(adl);
    CustomFactory cf = (CustomFactory) cl.newInstance();
    cf.setFcGenericFactory(gf);
    cf.setFcTypeFactory(tf);
    Component membrane = cf.newFcInstance();
    Component component = gf.newFcInstance(cType, membrane, componentClass);
    
		return component;
		
	}

  /**
   * 
   * 
   * 
   * @param membrane
   * @param comp
   * @throws NoSuchInterfaceException 
   */
	private static void setAmbientController(Component membrane) throws NoSuchInterfaceException {
		
		
		//System.err.println("  Sub-components (control components)");
    Component[] subs = FractalHelper.getAllSubComponents(membrane);
    
    
    // getting the AmbientController component from the list of membrane's subcomponents
    // start at index 1 to skip c
    //int i = 1;
    //for (i = 1; i < subs.length; i++) {
      //if (subs[i].getClass().equals(ambient.AmbientControllerImpl)) 
      //	break;
    	
    	//<TODO> which subcomponent it is?
    	//Class cl = subs[i].getClass();
    	//Type tp = subs[i].getFcType();
    	//System.out.println("..");
    //}
		
    
    
    
    
    
    //setting the reference from the 'content' to the 'membraneComponent', which means
    //                                       from AmbientControllerImpl to the AmbientController  
    Component ambientController = subs[1];
    AmbientControllerImpl ac = (AmbientControllerImpl) ambientController.getFcInterface("/content");
    ac.setComponent(ambientController);
    
    
    //<TODO> add attributeInterface to the AmbientController component.... and set it 
    //((IAttributeManager)Fractal.getAttributeController(ambientController)).setComponent(ambientController);
    //atc.setComponent(ambientController);
		
    
	}
	
	
	
	
  /**
   * Display a description of the given component: interfaces,
   * sub-components and bindings.
   * 
   * @param c  a component
   * @throws NoSuchInterfaceException 
   */
  private static void display( Component c ) throws NoSuchInterfaceException {
      
      System.err.println("  Interfaces");
      Object[] itfs = c.getFcInterfaces();
      for (int i = 0; i < itfs.length; i++) {
          Interface itf = (Interface) itfs[i];
          InterfaceType it = (InterfaceType) itf.getFcItfType();
          System.err.println("  "+i+": "+it.toString());
      }
      System.err.println();
      
      
      
      System.err.println("  Sub-components (control components)");
      Component[] subs = FractalHelper.getAllSubComponents(c);
      // start at index 1 to skip c
      for (int i = 1; i < subs.length; i++) {
          String name = FractalHelper.getNameController(subs[i]).getFcName();
          System.err.println("  "+(i-1)+": "+name);
      }
      
      
      
  }
	
  
  
  
  /**
   * Display a description of the given component: interfaces,
   * sub-components and bindings.
   * 
   * @param c  a component
   * @throws NoSuchInterfaceException 
   * @throws IllegalLifeCycleException 
   * @throws IllegalBindingException 
   */
  private static void myDisplay( Component membrane, Component pdaComp ) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
  	
  	Component[] subs = FractalHelper.getAllSubComponents(membrane);
    Component comp = subs[1];
    Interface itf = (Interface) comp.getFcInterface("iBindingController");
    System.err.println(" interface: "+ itf);
    System.err.println(" interface: "+ membrane.getFcInterface("//binding-controller"));
    
    
    BindingController bc = (BindingController) membrane.getFcInterface("//binding-controller");
    bc.listFc();
    
    bc = (BindingController) itf;
    bc.listFc();
    
    if(itf.equals(membrane.getFcInterface("//binding-controller")) )
    		System.err.println(" binding controller reference se rovnaji: ");
    else
    	System.err.println("nooo ");
    
    
		
  }
  

	
	
	
}
