package run;



import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.juliak.FractalHelper;
import org.objectweb.fractal.juliak.control.membrane.MembraneController;


import org.objectweb.fractal.util.Fractal;

import ambient.AmbientControllerInterface;
import ambient.C;

import pda.IAttributeManager;
import pda.IMain;
import printer.IPrinter;


public class AmbientRunner {

	/**
	 * @param args
	 * @throws InstantiationException 
	 * @throws NoSuchInterfaceException 
	 * @throws IllegalLifeCycleException 
	 * @throws IllegalContentException 
	 * @throws IllegalBindingException 
	 */
	public static void main(String[] args) throws Exception {
		
		Component mainComp = null;
		Component pdaComp = null;
		Component printerComp = null;
		
		Component registryComp = null;
		
		//BootStraping
		Component boot = Fractal.getBootstrapComponent();
    TypeFactory tf = Fractal.getTypeFactory(boot);
    GenericFactory gf = Fractal.getGenericFactory(boot);
     
		
		
    //	 -------------------------------------------------------------------
    //  1. creating component types
    // -------------------------------------------------------------------
		
		
	
		ComponentType mainType = tf.createFcType( new InterfaceType[]
		           {tf.createFcItfType("iMain","pda.IMain",false,false,false)}
		);
		
		ComponentType cType = tf.createFcType( new InterfaceType[]{ 
				tf.createFcItfType("iMain","pda.IMain",false,false,false),
				tf.createFcItfType("iPrinter","printer.IPrinter",true,true,false),  // ambient reference!
				tf.createFcItfType("iRegistry","registry.IRegistry",true,false,false),
				//tf.createFcItfType("attribute-controller","pda.IManagerAttribute",false,false,false),
				tf.createFcItfType("attribute-controller","pda.IAttributeManager",false,false,false),
				tf.createFcItfType("iAmbientController","run.IAmbientController",false,false,false)
						
		});
		                                      		
		ComponentType sType = tf.createFcType( new InterfaceType[]{ 
				tf.createFcItfType("iPrinter","printer.IPrinter",false,false,false),
				tf.createFcItfType("iRegistry","registry.IRegistry",true,false,false),
				tf.createFcItfType("attribute-controller","pda.IAttributeManager",false,false,false)	
		});
				
		ComponentType registryType = tf.createFcType( new InterfaceType[]{ 
				tf.createFcItfType("iRegistry","registry.IRegistry",false,false,false),
		}); 
	
		
		//			 -------------------------------------------------------------------
    // 2. CREATE COMPONENTS DIRECTLY
    // -------------------------------------------------------------------

    // create root component
		mainComp = gf.newFcInstance(mainType, "composite", null);

    

    // create server component
    printerComp = gf.newFcInstance(sType, "primitive", printer.PrinterImpl.class.getName());
    ((IAttributeManager)Fractal.getAttributeController(printerComp)).setComponent(printerComp);
    
       
		
		pdaComp = createAmbientClient(cType,pda.PDAImpl.class.getName(),gf);
    // create client component
    //pdaComp = gf.newFcInstance(cType, "primitive", "pda.PDAImpl");
    //((IManagerAttribute)Fractal.getAttributeController(pdaComp)).setComponent(pdaComp);

       
    

    //creating registry Component
    registryComp = gf.newFcInstance(registryType, "primitive", "registry.RegistryImpl");
    
   
    
    // -------------------------------------------------------------------
    // 3. component Assembly
    // -------------------------------------------------------------------

    
    Fractal.getContentController(mainComp).addFcSubComponent(pdaComp);
    Fractal.getContentController(mainComp).addFcSubComponent(printerComp);
    Fractal.getContentController(mainComp).addFcSubComponent(registryComp);
    Fractal.getBindingController(mainComp).bindFc("iMain", pdaComp.getFcInterface("iMain"));
    
    
    //binding to Registry
    Fractal.getBindingController(pdaComp).bindFc("iRegistry", registryComp.getFcInterface("iRegistry"));
    Fractal.getBindingController(printerComp).bindFc("iRegistry", registryComp.getFcInterface("iRegistry"));
    
       
    
    
    
    // -------------------------------------------------------------------
    // Membrane magic
    // -------------------------------------------------------------------
    
    Object itf = pdaComp.getFcInterface("/membrane-controller");
    MembraneController mc = (MembraneController) itf;
    Component membrane = mc.getFcMembrane();
    display(membrane);
    display(pdaComp);
    
    
    
    
    Interface itf1 =  (Interface) membrane.getFcInterface("//binding-controller");
    
    BindingController bc = (BindingController) itf1;
    
    ContentController cc ;
    
    
    
    //bc.bindFc("iRegistry", registryComp.getFcInterface("iRegistry"));
    //String[] l = bc.listFc();
        
    //(IBindingController) bci =  (IBindingController) itf1;
    
    
       
    
    // ambient testing
    AmbientControllerInterface ambientController =  (AmbientControllerInterface) membrane.getFcInterface("//ambient-controller");
    ambientController.bindAmbientFc(null, null);
    
   
  		
		
		// -------------------------------------------------------------------
    // 3. RUNNING
    // -------------------------------------------------------------------

    
		//LifeController
		((LifeCycleController) mainComp.getFcInterface("lifecycle-controller")).startFc();
	
		    
		//initialization of the components
	  ((IMain) mainComp.getFcInterface("iMain")).init();
	  ((IPrinter) printerComp.getFcInterface("iPrinter")).init();
		
	  
	  System.out.println("\n\n...running");
     //	 and run 
	  ((IMain) mainComp.getFcInterface("iMain")).main(args);
		
		System.out.println("\n\n");
		
		
		
		
		
		
		
		
		
		
		
		
		
		//	 -------------------------------------------------------------------
    // 4. Another TESTING
    // -------------------------------------------------------------------

		
		
    /////////////////////////////////////////////////////////////////////////////
    
    // stoping the printer
	  ((IPrinter) printerComp.getFcInterface("iPrinter")).stop();
		
	  System.out.println("\n\n");
	  
	  //	 and run 
	  ((IMain) mainComp.getFcInterface("iMain")).main(args);
		
		
		
	  
	  ((IPrinter) printerComp.getFcInterface("iPrinter")).start();
    
   
    
    
    
    
    
    /////////////////////////////////////////////////////////////////////////////
   
   
    //
    // Re-invoking the component.
    //
    //((Runnable)itf).run();
    //((IMain) mainComp.getFcInterface("iMain")).main(args);
    

    
    
    
    
    
    /////////////////////////////////////////////////////////////////////////////
    
    
    
		 
		 
	}
	
	
	
	
	
	 
  /////////////////////////////////////////////////////////////////////////////
  


	/**
	 * 
	 * @param type
	 * @param name
	 * @param gf 
	 * @return
	 * @throws InstantiationException 
	 * @throws NoSuchInterfaceException 
	 * @throws ADLException 
	 */
	private static Component createAmbientClient(ComponentType type, String name, GenericFactory gf) throws InstantiationException, NoSuchInterfaceException, ADLException {
		
		
		//create membrane
		Component ambientMembrane = getAmbientMembrane();
				
		// create client component
    Component pdaComp = gf.newFcInstance(type, ambientMembrane , pda.PDAImpl.class.getName());
    ((IAttributeManager)Fractal.getAttributeController(pdaComp)).setComponent(pdaComp);
		
    
    return pdaComp;
   
        
	}

	
	
	

  /**
   * 
   * Creates membrane with AmbientController.
   * 
   * 
   * @return
   * @throws ADLException 
   */
	private static Component getAmbientMembrane() throws ADLException {
		
		Factory f = FactoryFactory.getFactory();
		Component ambientMembrane = (Component) f.newComponent("ambient.AmbientPrimitive", null);
		
		
		
		
		
		//Component ambientMembrane = (Component) f.newComponent("org.objectweb.fractal.juliak.membrane.Primitive", null);
		
		
		return ambientMembrane;
	}


		
	
	
	
	
	
	
	
	
	 
  /////////////////////////////////////////////////////////////////////////////
  
	
	
	
	
  /**
   * Display a description of the given component: interfaces,
   * sub-components and bindings.
   * 
   * @param c  a component
   */
  private static void display( Component c ) {
      
      System.err.println("  Interfaces");
      Object[] itfs = c.getFcInterfaces();
      for (int i = 0; i < itfs.length; i++) {
          Interface itf = (Interface) itfs[i];
          InterfaceType it = (InterfaceType) itf.getFcItfType();
          System.err.println("  "+i+": "+it.toString());
      }
      System.err.println();
      
      
      
      System.err.println("  Sub-components (control components)");
      Component[] subs = FractalHelper.getAllSubComponents(c);
      // start at index 1 to skip c
      for (int i = 1; i < subs.length; i++) {
          String name = FractalHelper.getNameController(subs[i]).getFcName();
          System.err.println("  "+(i-1)+": "+name);
      }
  }
	
  
  /**
   * Display a description of the given component: interfaces,
   * sub-components and bindings.
   * 
   * @param c  a component
   */
  private static void myDisplay( Component c ) {
    
  	
  	System.err.println("\n\n\n\n << Printing the membrane components");
    
  	
  	MembraneController mc;
		try {
			mc = (MembraneController) c.getFcInterface("/membrane-controller");
		
  	  Component membrane = mc.getFcMembrane();
  	  //membrane.
  	
      System.err.println("  Interfaces");
      Object[] itfs = membrane.getFcInterfaces();
      for (int i = 0; i < itfs.length; i++) {
          Interface itf = (Interface) itfs[i];
          InterfaceType it = (InterfaceType) itf.getFcItfType();
          System.err.println("  "+i+": "+it.toString());
      }
      System.err.println();
      
      
      
      System.err.println("  Sub-components (control components)");
      Component[] subs = FractalHelper.getAllSubComponents(membrane);
      // start at index 1 to skip c
      for (int i = 1; i < subs.length; i++) {
          String name = FractalHelper.getNameController(subs[i]).getFcName();
          System.err.println("  "+(i-1)+": "+name);
      }
      
      
      
      
      
      
      
		} catch (NoSuchInterfaceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
  }
  
  
}
